;---------------------------------------------------------------------
; ASSEMBLY LANGUAGE GEM APPLICATION SHELL
;
; DEVELOPED WITH DEVPAC2 ST.
; COPYRIGHT 1992 BY CLAYTON WALNUM
;---------------------------------------------------------------------

PTERM0          equ     0
MSHRINK         equ     74

APPL_INIT       equ     10
GRAF_HANDLE     equ     77
APPL_EXIT       equ     19
V_CLRWK         equ     3
V_OPNVWK        equ     100
V_CLSVWK        equ     101

AES_OPCODE      equ     200
VDI_OPCODE      equ     115

;--------------------------------------------------------------------
; MAIN PROGRAM
;--------------------------------------------------------------------
        text

; Calculate the size of the program area.

        move.l  a7,a5                   ; Save addr of TPA.
        lea     stack,sp                ; Load addr of our stack.
        
        move.l  4(a5),a5                ; Get addr of TPA.              
        move.l  12(a5),d0               ; Get len of text segment.
        add.l   20(a5),d0               ; Add len of data segment.
        add.l   28(a5),d0               ; Add len of BSS segment.
        add.l   #$100,d0                ; Add len of TPA. 

; Release unused memory back to the system.
        
        move.l  d0,-(sp)                ; Push size of program on stack.
        move.l  a5,-(sp)                ; Push program addr on stack.
        clr     -(sp)                   ; Clear dummy word on stack.    
        move    #MSHRINK,-(sp)          ; Push Mshrink opcode.
        trap    #1                      ; Call GEMDOS.
        add.l   #12,sp                  ; Reset stack pointer.

; Clear some fields of the global array.

        clr.l   ap_ptree        
        clr.l   ap_1resv
        clr.l   ap_2resv
        clr.l   ap_3resv
        clr.l   ap_4resv

; Call appl_init to initialize application.

        move    #APPL_INIT,control
        move    #0,control+2
        move    #1,control+4
        move    #0,control+6
        move    #0,control+8
        bsr     aes
        cmpi    #$FFFF,ap_id            ; Error?
        beq     end                     ; Yep.  Outta here.

; Get handle to screen device.

        move    #GRAF_HANDLE,control
        move    #0,control+2
        move    #5,control+4
        move    #0,control+6
        move    #0,control+8
        jsr     aes
        move    int_out,gr_handle       ; Save device handle.

; Open virtual workstation.

        move    #V_OPNVWK,control
        clr     control+2
        move    #11,control+6
        move    gr_handle,control+12    
        movea.l #int_in,a5              ; Get address of int_in.
        move    #9,d5                   ; Init loop counter.
loop:
        move    #1,(a5)+                ; Init int_in array.
        dbra    d5,loop
        move    #2,int_in+20
        jsr     vdi
        move    control+12,vws_handle   ; Save virtual device handle.

;------------------------------------------------------
; YOUR PROGRAM CODE GOES HERE
;------------------------------------------------------

; Close the virtual workstation.

        move    #V_CLSVWK,control
        clr     control+2
        clr     control+6
        move    vws_handle,control+12
        jsr     vdi
                
; Close down application.

exit:
        move    #APPL_EXIT,control
        move    #0,control+2
        move    #1,control+4
        move    #0,control+6
        move    #0,control+8
        bsr     aes
        
end:
        move.w  #PTERM0,-(sp)           ; Back to desktop.
        trap    #1

;--------------------------------------------------------------------
; This subroutine calls the AES.  Before calling this subroutine, the
; program must have correctly initialized the AES control, int_in,
; and addr_in arrays.
;
; Input:        Appropriate values in the int_in, addr_in, and
;               control arrays.
; Output:       Appropriate values in the int_out, addr_out, and
;               global arrays.
; Regs changed: NONE
; Uses: apb, global, int_in, int_out, addr_in, addr_out
;--------------------------------------------------------------------
aes:
        movem.l a0-a7/d0-d7,-(sp)       ; Save registers.
        move.l  #apb,d1                 ; Load addr of apb.
        move.w  #AES_OPCODE,d0          ; Load AES opcode.
        trap    #2                      ; Call AES.
        movem.l (sp)+,a0-a7/d0-d7       ; Restore registers.
        rts

;--------------------------------------------------------------------
; This subroutine calls the VDI. Before calling this subroutine, the
; program must have correctly initialized the VDI control, intin,
; and ptsin arrays.
;
; Input:        Appropriate values in the intin, ptsin, and
;               control arrays.
; Output:       Appropriate values in the intout and ptsout arrays.
; Regs changed: NONE
; Uses: vpb, intin, intout, ptsin, ptsout
;--------------------------------------------------------------------
vdi:
        movem.l a0-a7/d0-d7,-(sp)       ; Save registers.
        move.l  #vpb,d1                 ; Load addr of vpb.
        move.w  #VDI_OPCODE,d0          ; Load VDI opcode.
        trap    #2                      ; Call VDI.
        movem.l (sp)+,a0-a7/d0-d7       ; Restore registers.
        rts

        data
        
        even

apb:            dc.l    control,global,int_in,int_out,addr_in,addr_out
vpb:            dc.l    control,int_in,pts_in,int_out,pts_out

        bss

        even

gr_handle       ds.w    1
vws_handle      ds.w    1
        
global:
ap_version:     ds.w    1
ap_count:       ds.w    1
ap_id:          ds.w    1
ap_private:     ds.l    1
ap_ptree:       ds.l    1
ap_1resv:       ds.l    1
ap_2resv:       ds.l    1
ap_3resv:       ds.l    1
ap_4resv:       ds.l    1

control:        ds.w    12
int_in:         ds.w    128
int_out:        ds.w    128
addr_in:        ds.l    64
addr_out:       ds.l    64
pts_in:         ds.w    128
pts_out:        ds.w    128

                ds.l    255
stack:          ds.l    1

